(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.20 2014/01/21 09:21:18 fclement Exp $ *)

open Skl;;

let rec read_lines ic ob =
  let buf = String.create 1024 in
  let len = input ic buf 0 1024 in
  if len = 0 then raise End_of_file else
  Buffer.add_substring ob buf 0 len;
  read_lines ic ob
;;

let eval_command cmd args =
  let cmd_line = cmd ^ " " ^ args in
  let ic = Unix.open_process_in cmd_line in
  let ob = Buffer.create 1024 in
  try read_lines ic ob with
  | End_of_file ->
    let status = Unix.close_process_in ic in
    let output = Buffer.contents ob in
    Buffer.clear ob;
    (args, status, output)
;;

(* The atomic eval command skeleton. *)
let eval_command_skl = skl cmd -> eval_command cmd;;

(* The full computational skeleton expression. *)
let parexec_skl nw cmd = farm (eval_command_skl cmd, nw);;

let open_file = function
  | "-" -> stdin
  | f -> open_in f
;;

let print_result cmd (args, status, output) =
  Printf.printf "%s" output;
  match status with
  | Unix.WEXITED n ->
      if n <> 0 then
        Printf.eprintf
          "Command '%s' with arguments '%s' failed with status %d.\n"
          cmd args n
  | Unix.WSIGNALED n | Unix.WSTOPPED n ->
      Printf.eprintf
        "Command '%s' with arguments '%s' was killed/stopped by signal %d.\n"
        cmd args n
;;

let main { Parfun.parfun = parfun; } =
  Ucommand_option.parse ();
  let command = Options.get_command () in
  let ic = open_file (Options.get_command_args ()) in
  let nw = Options.get_number_of_workers () in
  let input_stream = Sklstream.of_channel_by_lines ic in
  let output_stream = parfun (parexec_skl nw command) input_stream in
  Sklstream.iter (print_result command) output_stream
;;

Options.add_options ();;
Parfun.pardo main;;
