(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2007-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: options.ml,v 1.13 2014/01/21 09:21:18 fclement Exp $ *)

let { Uaref.get = get_number_of_workers; set = set_number_of_workers; } =
  Uaref.make_aref_gen
    (Uaref.with_int_bounds (Some 1) None
      (Uaref.with_name "number_of_workers"
        (Uaref.with_single_assignment
          (Uaref.aref_spec 2))))

and { Uaref.get = get_command; set = set_command; } =
  Uaref.make_aref_single "command" "echo"

and { Uaref.get = get_command_args; set = set_command_args; } =
  Uaref.make_aref_single "command_args" "-"
;;

let add_options () =
  Ucommand_option.user_add_all
    [
     Ucommand_option.with_short_key "-nw"
       (Ucommand_option.of_regular
          {
           Ucommand_option.key = "--number-of-workers";
           spec = Arg.Int set_number_of_workers;
           doc =
             Printf.sprintf
               "<nw>  Set the number of effective workers.\
               \n\tDefault is %d."
               (get_number_of_workers ());
          });
     
     Ucommand_option.with_auto_short_key
       (Ucommand_option.of_regular
          {
           Ucommand_option.key = "--command-args";
           spec = Arg.String set_command_args;
           doc =
             Printf.sprintf
               "<file>  Set the file containing arguments of the command\
               \n\tto run in parallel. \
               Use '-' (dash) to specify the standard input.\
               \n\tDefault is %S."
               (get_command_args ());
          });
     
     Ucommand_option.with_auto_short_key
       (Ucommand_option.of_regular
          {
           Ucommand_option.key = "--command";
           spec = Arg.String set_command;
           doc =
             Printf.sprintf
               "<cmd>  Set the command to run. \
               Default is %S."
               (get_command ());
          });
    ];
;;

Ucommand_option.set_usage_msg
  (Printf.sprintf "\n%s -- Parallel execution of a command\n" Sys.argv.(0))
;;
